@@ -1,5 +1,42 @@
-#include <lqt.h>
-#include <qtprivate.h>
+/*******************************************************************************
+ lqt_funcprotos.h
+
+ libquicktime - A library for reading and writing quicktime/avi/mp4 files.
+ http://libquicktime.sourceforge.net
+
+ Copyright (C) 2002 Heroine Virtual Ltd.
+ Copyright (C) 2002-2007 Members of the libquicktime project.
+
+ This library is free software; you can redistribute it and/or modify it under
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2.1 of the License, or (at your option)
+ any later version.
+
+ This library is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+ details.
+
+ You should have received a copy of the GNU Lesser General Public License along
+ with this library; if not, write to the Free Software Foundation, Inc., 51
+ Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*******************************************************************************/
+#include <config.h>
+#include <quicktime/lqt.h>
+#include <quicktime/qtprivate.h>
+
+#ifdef HAVE_GCC_VISIBILITY
+#define LQT_EXTERN __attribute__ ((visibility("default")))
+#else
+#define LQT_EXTERN extern
+#endif
+
+/* Some macros */
+
+#define IS_QUICKTIME(type) (type & (LQT_FILE_QT_OLD | LQT_FILE_QT))
+#define IS_MP4(type) (type & (LQT_FILE_MP4 | LQT_FILE_M4A | LQT_FILE_3GP))
+#define IS_AVI(type) (type & (LQT_FILE_AVI | LQT_FILE_AVI_ODML))
+
 /* atom.c */
 
 int quicktime_atom_read_header(quicktime_t *file, quicktime_atom_t *atom);
@@ -11,11 +48,11 @@ int quicktime_atom_skip(quicktime_t *file, quicktime_atom_t *atom);
 
 /* audio.c */
 
-void lqt_convert_audio_encode(int16_t ** in_int, float ** in_float,
+void lqt_convert_audio_encode(quicktime_t * file, int16_t ** in_int, float ** in_float,
                               void * out, int num_channels, int num_samples,
                               lqt_sample_format_t stream_format);
 
-void lqt_convert_audio_decode(void * in, int16_t ** out_int,
+void lqt_convert_audio_decode(quicktime_t * file, void * in, int16_t ** out_int,
                               float ** out_float, int num_channels, int num_samples,
                               lqt_sample_format_t stream_format);
  
@@ -71,6 +108,9 @@ void quicktime_update_idx1table(quicktime_t *file,
                                 int offset,
                                 int size);
 
+void quicktime_idx1_dump(quicktime_idx1_t *idx1);
+
+
 /* avi_indx.c */
 
 void quicktime_finalize_indx(quicktime_t *file, quicktime_indx_t *indx);
@@ -178,7 +218,7 @@ void quicktime_strf_dump_video(quicktime_strf_t *strf);
 void quicktime_strf_set_video_extradata(quicktime_strf_t *strf, const uint8_t * data, int len);
 void quicktime_strf_delete_video(quicktime_strf_t *strf);
 
-void quicktime_strf_set_audio_extradata(quicktime_strf_t *strf, const uint8_t * data, int len);
+LQT_EXTERN void quicktime_strf_set_audio_extradata(quicktime_strf_t *strf, const uint8_t * data, int len);
 void quicktime_strf_delete_audio(quicktime_strf_t *strf);
 
 
@@ -235,7 +275,7 @@ void quicktime_write_chan(quicktime_t *file, quicktime_chan_t *chan);
 void quicktime_get_chan(quicktime_audio_map_t * atrack);
 
 /* Set the chan atom of an atrack */
-void quicktime_set_chan(quicktime_audio_map_t * atrack);
+LQT_EXTERN void quicktime_set_chan(quicktime_audio_map_t * atrack);
 
 /* clap.c */
 
@@ -304,26 +344,26 @@ void quicktime_read_ctts(quicktime_t *file, quicktime_ctts_t *ctts);
 void quicktime_write_ctts(quicktime_t *file, quicktime_ctts_t *ctts);
 void quicktime_update_ctts(quicktime_ctts_t *ctts, long sample, long duration);
 void quicktime_compress_ctts(quicktime_ctts_t *ctts);
-
+void quicktime_fix_ctts(quicktime_ctts_t *ctts);
 
 /* dinf.c */
 
 void quicktime_dinf_init(quicktime_dinf_t *dinf);
 void quicktime_dinf_delete(quicktime_dinf_t *dinf);
-void quicktime_dinf_init_all(quicktime_dinf_t *dinf);
+void quicktime_dinf_init_all(quicktime_dinf_t *dinf, lqt_file_type_t type);
 void quicktime_dinf_dump(quicktime_dinf_t *dinf);
 void quicktime_read_dinf(quicktime_t *file, quicktime_dinf_t *dinf, quicktime_atom_t *dinf_atom);
 void quicktime_write_dinf(quicktime_t *file, quicktime_dinf_t *dinf);
 
 /* dref.c */
 
-void quicktime_dref_table_init(quicktime_dref_table_t *table);
+void quicktime_dref_table_init(quicktime_dref_table_t *table, lqt_file_type_t type);
 void quicktime_dref_table_delete(quicktime_dref_table_t *table);
 void quicktime_read_dref_table(quicktime_t *file, quicktime_dref_table_t *table);
 void quicktime_write_dref_table(quicktime_t *file, quicktime_dref_table_t *table);
 void quicktime_dref_table_dump(quicktime_dref_table_t *table);
 void quicktime_dref_init(quicktime_dref_t *dref);
-void quicktime_dref_init_all(quicktime_dref_t *dref);
+void quicktime_dref_init_all(quicktime_dref_t *dref, lqt_file_type_t type);
 void quicktime_dref_delete(quicktime_dref_t *dref);
 void quicktime_dref_dump(quicktime_dref_t *dref);
 void quicktime_read_dref(quicktime_t *file, quicktime_dref_t *dref);
@@ -336,7 +376,7 @@ void quicktime_edts_delete(quicktime_edts_t *edts);
 void quicktime_edts_init_table(quicktime_edts_t *edts);
 void quicktime_read_edts(quicktime_t *file, quicktime_edts_t *edts, quicktime_atom_t *edts_atom);
 void quicktime_edts_dump(quicktime_edts_t *edts);
-void quicktime_write_edts(quicktime_t *file, quicktime_edts_t *edts, long duration);
+void quicktime_write_edts(quicktime_t *file, quicktime_edts_t *edts);
 
 /* enda.c */
 
@@ -345,24 +385,24 @@ void quicktime_read_enda(quicktime_t *file, quicktime_enda_t *enda,
 void quicktime_write_enda(quicktime_t *file, quicktime_enda_t *enda);
 void quicktime_enda_dump(quicktime_enda_t *enda);
 
-void quicktime_set_enda(quicktime_stsd_table_t *table, int little_endian);
+LQT_EXTERN void quicktime_set_enda(quicktime_stsd_table_t *table, int little_endian);
 
 /* Returns TRUE if little endian */
-int quicktime_get_enda(quicktime_stsd_table_t *table);
+LQT_EXTERN int quicktime_get_enda(quicktime_stsd_table_t *table);
 
 /* elst.c */
 
 void quicktime_elst_table_init(quicktime_elst_table_t *table);
 void quicktime_elst_table_delete(quicktime_elst_table_t *table);
 void quicktime_read_elst_table(quicktime_t *file, quicktime_elst_table_t *table);
-void quicktime_write_elst_table(quicktime_t *file, quicktime_elst_table_t *table, long duration);
+void quicktime_write_elst_table(quicktime_t *file, quicktime_elst_table_t *table);
 void quicktime_elst_table_dump(quicktime_elst_table_t *table);
 void quicktime_elst_init(quicktime_elst_t *elst);
 void quicktime_elst_init_all(quicktime_elst_t *elst);
 void quicktime_elst_delete(quicktime_elst_t *elst);
 void quicktime_elst_dump(quicktime_elst_t *elst);
 void quicktime_read_elst(quicktime_t *file, quicktime_elst_t *elst);
-void quicktime_write_elst(quicktime_t *file, quicktime_elst_t *elst, long duration);
+void quicktime_write_elst(quicktime_t *file, quicktime_elst_t *elst);
 
 /* esds.c */
 
@@ -379,7 +419,7 @@ int quicktime_read_mp4_descr_length(quicktime_t * file);
    and decoderConfigLen. The esds atom will be returned so the
    other parameters can be set by the encoder */
 
-quicktime_esds_t * quicktime_set_esds(quicktime_trak_t * trak,
+LQT_EXTERN quicktime_esds_t * quicktime_set_esds(quicktime_trak_t * trak,
                                       uint8_t * decoderConfig,
                                       int decoderConfigLen);
 
@@ -399,7 +439,16 @@ void quicktime_read_frma(quicktime_t *file, quicktime_frma_t *frma,
 
 void quicktime_write_frma(quicktime_t *file, quicktime_frma_t *frma);
 void quicktime_frma_dump(quicktime_frma_t *frma);
-void quicktime_set_frma(quicktime_trak_t * trak, char * codec);
+LQT_EXTERN void quicktime_set_frma(quicktime_trak_t * trak, char * codec);
+
+/* ftab.c */
+
+int quicktime_ftab_init(quicktime_ftab_t *ftab, int font_id, const char * font_name);
+int quicktime_ftab_delete(quicktime_ftab_t *ftab);
+void quicktime_ftab_dump(quicktime_ftab_t *ftab);
+int quicktime_read_ftab(quicktime_t *file, quicktime_ftab_t *ftab);
+void quicktime_write_ftab(quicktime_t *file, quicktime_ftab_t *ftab);
+
 
 /* ftyp.c */
 
@@ -411,7 +460,8 @@ lqt_file_type_t quicktime_ftyp_get_file_type(quicktime_ftyp_t * ftyp);
 void quicktime_read_ftyp(quicktime_t *file, quicktime_ftyp_t *ftyp,
                          quicktime_atom_t *parent_atom);
 void quicktime_write_ftyp(quicktime_t *file, quicktime_ftyp_t *ftyp);
-
+void quicktime_ftyp_dump(quicktime_ftyp_t *ftyp);
+void quicktime_ftyp_delete(quicktime_ftyp_t *ftyp);
 
 /* gama.c */
 
@@ -429,6 +479,20 @@ void quicktime_gmhd_delete(quicktime_gmhd_t *gmhd);
 void quicktime_gmhd_dump(quicktime_gmhd_t *gmhd);
 void quicktime_read_gmhd(quicktime_t *file, quicktime_gmhd_t *gmhd, quicktime_atom_t *parent_atom);
 void quicktime_write_gmhd(quicktime_t *file, quicktime_gmhd_t *gmhd);
+void quicktime_gmhd_init_timecode(quicktime_gmhd_t *gmhd);
+
+
+/* gmhd_text.c */
+
+void quicktime_gmhd_text_init(quicktime_gmhd_text_t *gmhd);
+void quicktime_gmhd_text_delete(quicktime_gmhd_text_t *gmhd);
+void quicktime_gmhd_text_dump(quicktime_gmhd_text_t *gmhd);
+void quicktime_read_gmhd_text(quicktime_t *file,
+                              quicktime_gmhd_text_t *gmhd,
+                              quicktime_atom_t *parent_atom);
+void quicktime_write_gmhd_text(quicktime_t *file,
+                               quicktime_gmhd_text_t *gmhd);
+
 
 /* gmin.c */
 
@@ -443,9 +507,14 @@ void quicktime_write_gmin(quicktime_t *file, quicktime_gmin_t *gmin);
 void quicktime_hdlr_init(quicktime_hdlr_t *hdlr);
 void quicktime_hdlr_init_video(quicktime_hdlr_t *hdlr);
 void quicktime_hdlr_init_panorama(quicktime_hdlr_t *hdlr);
-void quicktime_hdlr_init_qtvr(quicktime_hdlr_t *hdlr, int track_type);
+int quicktime_hdlr_init_qtvr(quicktime_hdlr_t *hdlr, int track_type);
 void quicktime_hdlr_init_audio(quicktime_hdlr_t *hdlr);
 void quicktime_hdlr_init_data(quicktime_hdlr_t *hdlr);
+void quicktime_hdlr_init_text(quicktime_hdlr_t *hdlr);
+void quicktime_hdlr_init_tx3g(quicktime_hdlr_t *hdlr);
+void quicktime_hdlr_init_timecode(quicktime_hdlr_t *hdlr);
+
+
 void quicktime_hdlr_init_udta(quicktime_hdlr_t *hdlr);
 void quicktime_hdlr_delete(quicktime_hdlr_t *hdlr);
 void quicktime_hdlr_dump(quicktime_hdlr_t *hdlr);
@@ -477,6 +546,26 @@ void quicktime_write_iods(quicktime_t *file, quicktime_moov_t * moov);
 void quicktime_iods_dump(quicktime_iods_t * iods);
 void quicktime_iods_delete(quicktime_iods_t * iods);
 
+/* language.c */
+
+const char * lqt_get_charset(int mac_code, lqt_file_type_t file_type);
+
+/* Macintosh charsets are not supported everywhere, but some are pretty
+   close to other ones */
+const char * lqt_get_charset_fallback(int mac_code, lqt_file_type_t file_type);
+
+/* log.c */
+
+LQT_EXTERN void lqt_log(quicktime_t * file, lqt_log_level_t level,
+             const char * domain, const char * format, ...)
+  __attribute__ ((format (printf, 4, 5)));
+
+LQT_EXTERN void lqt_logs(quicktime_t * file, lqt_log_level_t level,
+              const char * domain, const char * str);
+
+/* For dumping strings */
+void lqt_dump(const char * format, ...)  __attribute__ ((format (printf, 1, 2)));
+
 /* matrix.c */
 
 void quicktime_matrix_init(quicktime_matrix_t *matrix);
@@ -500,13 +589,18 @@ void quicktime_mdhd_dump(quicktime_mdhd_t *mdhd);
 void quicktime_read_mdhd(quicktime_t *file, quicktime_mdhd_t *mdhd);
 void quicktime_write_mdhd(quicktime_t *file, quicktime_mdhd_t *mdhd);
 
-void quicktime_mdhd_init_audio(quicktime_mdhd_t *mdhd, 
+void quicktime_mdhd_init_audio(quicktime_t *file,
+                               quicktime_mdhd_t *mdhd, 
                                int sample_rate);
 
 void quicktime_mdhd_init_video(quicktime_t *file, 
                                quicktime_mdhd_t *mdhd,
                                int timescale);
 
+void quicktime_mdhd_init_text(quicktime_t *file,
+                              quicktime_mdhd_t *mdhd, 
+                              int timescale);
+
 /* mdia.c */
 
 void quicktime_mdia_init(quicktime_mdia_t *mdia);
@@ -541,10 +635,25 @@ void quicktime_mdia_init_panorama(quicktime_t *file,
 void quicktime_mdia_init_qtvr(quicktime_t *file,
 				  quicktime_mdia_t *mdia,
 				  int track_type,
-				  int width,
-				  int height,
 				  int frame_duration,
 				  int timescale);
+
+void quicktime_mdia_init_text(quicktime_t * file,
+                              quicktime_mdia_t * mdia, 
+                              int timescale);
+
+void quicktime_mdia_init_tx3g(quicktime_t * file,
+                              quicktime_mdia_t * mdia, 
+                              int timescale);
+
+void quicktime_mdia_init_timecode(quicktime_t *file,
+                                  quicktime_mdia_t *mdia,
+                                  int time_scale,
+                                  int frame_duration,
+                                  int num_frames,
+                                  uint32_t flags);
+
+
 /* minf.c */
 
 void quicktime_minf_init(quicktime_minf_t *minf);
@@ -579,10 +688,21 @@ void quicktime_minf_init_panorama(quicktime_t *file,
 void quicktime_minf_init_qtvr(quicktime_t *file,
 				  quicktime_minf_t *minf,
 				  int frame_type,
-				  int width,
-				  int height,
 				  int frame_duration);
 
+void quicktime_minf_init_text(quicktime_t *file, 
+                              quicktime_minf_t *minf);
+
+void quicktime_minf_init_tx3g(quicktime_t *file, 
+                              quicktime_minf_t *minf);
+
+void quicktime_minf_init_timecode(quicktime_t *file,
+                                  quicktime_minf_t *minf,
+                                  int time_scale,
+                                  int frame_duration,
+                                  int num_frames,
+                                  uint32_t flags);
+
 /* moov.c */
 
 int quicktime_moov_init(quicktime_moov_t *moov);
@@ -627,6 +747,15 @@ void quicktime_nloc_dump(quicktime_nloc_t *nloc);
 int quicktime_read_nloc(quicktime_t *file, quicktime_nloc_t *nloc, quicktime_qtatom_t *nloc_atom);
 void quicktime_write_nloc(quicktime_t *file, quicktime_nloc_t *nloc);
 
+/* nmhd.c */
+
+int quicktime_nmhd_init(quicktime_nmhd_t *nmhd);
+int quicktime_nmhd_delete(quicktime_nmhd_t *nmhd);
+void quicktime_nmhd_dump(quicktime_nmhd_t *nmhd);
+int quicktime_read_nmhd(quicktime_t *file, quicktime_nmhd_t *nmhd);
+void quicktime_write_nmhd(quicktime_t *file, quicktime_nmhd_t *nmhd);
+
+
 /* obji.c */
 
 int quicktime_obji_init(quicktime_obji_t *obji);  
@@ -662,6 +791,16 @@ void quicktime_write_pdat(quicktime_t *file, quicktime_pdat_t *pdat);
 /* pHdr */
 
 int quicktime_pHdr_init(quicktime_pHdr_t *pHdr);
+int quicktime_pHdr_init(quicktime_pHdr_t *pHdr);
+int quicktime_pHdr_delete(quicktime_pHdr_t *pHdr);
+void quicktime_pHdr_dump(quicktime_pHdr_t *pHdr);
+
+int quicktime_read_pHdr(quicktime_t *file,
+                        quicktime_pHdr_t *pHdr,
+                        quicktime_atom_t *pHdr_atom);
+
+void quicktime_write_pHdr(quicktime_t *file,
+                          quicktime_pHdr_t *pHdr);
 
 /* qtatom.c */
 
@@ -724,9 +863,21 @@ void quicktime_stbl_init_panorama(quicktime_t *file,
 void quicktime_stbl_init_qtvr(quicktime_t *file,
 				  quicktime_stbl_t *stbl,
 				  int track_type,
-				  int width,
-				  int height,
 				  int frame_duration);			     
+
+void quicktime_stbl_init_text(quicktime_t *file,
+                              quicktime_stbl_t *stbl);
+
+void quicktime_stbl_init_tx3g(quicktime_t *file,
+                              quicktime_stbl_t *stbl);
+
+void quicktime_stbl_init_timecode(quicktime_t *file,
+                                  quicktime_stbl_t *stbl,
+                                  int time_scale,
+                                  int frame_duration,
+                                  int num_frames,
+                                  uint32_t flags);
+
 /* stco.c */
 
 void quicktime_stco_init(quicktime_stco_t *stco);
@@ -749,6 +900,7 @@ void quicktime_stsc_dump(quicktime_stsc_t *stsc);
 void quicktime_read_stsc(quicktime_t *file, quicktime_stsc_t *stsc);
 void quicktime_write_stsc(quicktime_t *file, quicktime_stsc_t *stsc);
 int quicktime_update_stsc(quicktime_stsc_t *stsc, long chunk, long samples);
+void quicktime_compress_stsc(quicktime_stsc_t *stsc);
 
 /* stsd.c */
 
@@ -759,7 +911,6 @@ void quicktime_stsd_dump(void *minf_ptr, quicktime_stsd_t *stsd);
 void quicktime_read_stsd(quicktime_t *file, quicktime_stsd_t *stsd);
 void quicktime_write_stsd(quicktime_t *file, quicktime_minf_t *minf, quicktime_stsd_t *stsd);
 
-
 /* This function is called by the decoder after the complete minf atom is read (i.e. if we know
    for sure, what kind of stream it is. */
 
@@ -785,29 +936,42 @@ void quicktime_stsd_init_panorama(quicktime_t *file,
 				  int width,
 				  int height);
 
-void quicktime_stsd_init_qtvr(quicktime_t *file,
+int quicktime_stsd_init_qtvr(quicktime_t *file,
+                              quicktime_stsd_t *stsd,
+                              int track_type);
+
+void quicktime_stsd_init_text(quicktime_t *file,
+                              quicktime_stsd_t *stsd);
+
+void quicktime_stsd_init_tx3g(quicktime_t *file,
+                              quicktime_stsd_t *stsd);
+
+void quicktime_stsd_init_timecode(quicktime_t *file,
 				  quicktime_stsd_t *stsd,
-				  int track_type,
-				  int width,
-				  int height);
+				  int timescale,
+				  int frameduration,
+				  int numframes, uint32_t flags);
+
 /* stsdtable.c */
 
+
 void quicktime_read_stsd_audio(quicktime_t *file, quicktime_stsd_table_t *table,
                                quicktime_atom_t *parent_atom);
 void quicktime_write_stsd_audio(quicktime_t *file, quicktime_stsd_table_t *table);
 void quicktime_read_stsd_video(quicktime_t *file, quicktime_stsd_table_t *table,
                                quicktime_atom_t *parent_atom);
 
-void quicktime_set_stsd_audio_v1(quicktime_stsd_table_t *table,
+LQT_EXTERN void quicktime_set_stsd_audio_v1(quicktime_stsd_table_t *table,
                                  uint32_t samples_per_packet,
                                  uint32_t bytes_per_packet,
                                  uint32_t bytes_per_frame,
                                  uint32_t bytes_per_sample);
 
-void quicktime_set_stsd_audio_v2(quicktime_stsd_table_t *table,
-                                 uint32_t formatSpecificFlags,
-                                 uint32_t constBytesPerAudioPacket,
-                                 uint32_t constLPCMFramesPerAudioPacket);
+LQT_EXTERN void
+quicktime_set_stsd_audio_v2(quicktime_stsd_table_t *table,
+                            uint32_t formatSpecificFlags,
+                            uint32_t constBytesPerAudioPacket,
+                            uint32_t constLPCMFramesPerAudioPacket);
 
 void quicktime_write_stsd_video(quicktime_t *file, quicktime_stsd_table_t *table);
 void quicktime_read_stsd_table(quicktime_t *file, quicktime_minf_t *minf, quicktime_stsd_table_t *table);
@@ -822,8 +986,8 @@ void quicktime_stsd_audio_dump(quicktime_stsd_table_t *table);
 void quicktime_stsd_table_dump(void *minf_ptr, quicktime_stsd_table_t *table);
 void quicktime_write_stsd_table(quicktime_t *file, quicktime_minf_t *minf, quicktime_stsd_table_t *table);
 
-uint8_t * quicktime_stsd_get_user_atom(quicktime_trak_t * trak, char * name, uint32_t * len);
-void quicktime_stsd_set_user_atom(quicktime_trak_t * trak, char * name,
+LQT_EXTERN uint8_t * quicktime_stsd_get_user_atom(quicktime_trak_t * trak, char * name, uint32_t * len);
+LQT_EXTERN void quicktime_stsd_set_user_atom(quicktime_trak_t * trak, char * name,
                                   uint8_t * data, uint32_t len);
 
 
@@ -852,9 +1016,7 @@ void quicktime_write_stsz(quicktime_t *file, quicktime_stsz_t *stsz);
 void quicktime_update_stsz(quicktime_stsz_t *stsz, 
                            long sample, 
                            long sample_size);
-
-
-
+void quicktime_stsz_init_timecode(quicktime_stsz_t *stsz);
 
 /* stts.c */
 
@@ -862,11 +1024,16 @@ void quicktime_stts_init(quicktime_stts_t *stts);
 void quicktime_stts_init_table(quicktime_stts_t *stts);
 void quicktime_stts_init_video(quicktime_t *file, quicktime_stts_t *stts, int frame_duration);
 void quicktime_stts_init_audio(quicktime_t *file, quicktime_stts_t *stts, int sample_rate);
+void quicktime_stts_init_timecode(quicktime_t *file, quicktime_stts_t *stts);
+void quicktime_stts_init_qtvr(quicktime_t *file,
+                              quicktime_stts_t *stts, int frame_duration);
+
+
 void quicktime_stts_delete(quicktime_stts_t *stts);
 void quicktime_stts_dump(quicktime_stts_t *stts);
 void quicktime_read_stts(quicktime_t *file, quicktime_stts_t *stts);
 void quicktime_write_stts(quicktime_t *file, quicktime_stts_t *stts);
-void quicktime_update_stts(quicktime_stts_t *stts, long sample, long duration);
+LQT_EXTERN void quicktime_update_stts(quicktime_stts_t *stts, long sample, long duration);
 /* The following function kicks out redundant entries */
 void quicktime_compress_stts(quicktime_stts_t *stts);
 
@@ -880,9 +1047,19 @@ int64_t quicktime_sample_to_time(quicktime_stts_t *stts, int64_t sample,
                                  int64_t * stts_index, int64_t * stts_count);
 
 
+
+/* texttrack.c */
+
+void lqt_init_text_map(quicktime_t * file,
+                       quicktime_text_map_t * map, quicktime_trak_t * trak,
+                       int encode);
+
+void lqt_delete_text_map(quicktime_t * file,
+                         quicktime_text_map_t * map);
+
 /* tkhd.c */
 
-int quicktime_tkhd_init(quicktime_tkhd_t *tkhd);
+int quicktime_tkhd_init(quicktime_tkhd_t *tkhd, lqt_file_type_t type);
 int quicktime_tkhd_delete(quicktime_tkhd_t *tkhd);
 void quicktime_tkhd_dump(quicktime_tkhd_t *tkhd);
 void quicktime_read_tkhd(quicktime_t *file, quicktime_tkhd_t *tkhd);
@@ -890,9 +1067,15 @@ void quicktime_write_tkhd(quicktime_t *file, quicktime_tkhd_t *tkhd);
 void quicktime_tkhd_init_video(quicktime_t *file, quicktime_tkhd_t *tkhd, 
                                int frame_w, int frame_h);
 
+void quicktime_tkhd_init_timecode(quicktime_t *file,
+                                  quicktime_tkhd_t *tkhd,
+                                  int frame_w,
+                                  int frame_h);
+
+
 /* trak.c */
 
-int quicktime_trak_init(quicktime_trak_t *trak);
+int quicktime_trak_init(quicktime_trak_t *trak, lqt_file_type_t type);
 int quicktime_trak_init_video(quicktime_t *file, 
                               quicktime_trak_t *trak, 
                               int frame_w, 
@@ -916,6 +1099,21 @@ int quicktime_trak_init_qtvr(quicktime_t *file,
 				 int timescale,
 				 int frame_duration);
 
+int quicktime_trak_init_text(quicktime_t * file, quicktime_trak_t * trak,
+                             int timescale);
+
+int quicktime_trak_init_tx3g(quicktime_t * file, quicktime_trak_t * trak,
+                             int timescale);
+
+int quicktime_trak_init_timecode(quicktime_t *file, 
+                                 quicktime_trak_t *trak,
+                                 int time_scale,
+                                 int frame_duration,
+                                 int num_frames,
+                                 int frame_w,
+                                 int frame_h,
+                                 uint32_t flags);
+
 int quicktime_trak_init_panorama(quicktime_t *file,
 				 quicktime_trak_t *trak,
 				 int width,
@@ -928,17 +1126,24 @@ int quicktime_trak_dump(quicktime_trak_t *trak);
 quicktime_trak_t* quicktime_add_trak(quicktime_t *file);
 int quicktime_delete_trak(quicktime_moov_t *moov);
 int quicktime_read_trak(quicktime_t *file, quicktime_trak_t *trak, quicktime_atom_t *trak_atom);
-int quicktime_write_trak(quicktime_t *file, quicktime_trak_t *trak, long moov_time_scale);
+int quicktime_write_trak(quicktime_t *file, quicktime_trak_t *trak);
 int64_t quicktime_track_samples(quicktime_t *file, quicktime_trak_t *trak);
 long quicktime_sample_of_chunk(quicktime_trak_t *trak, long chunk);
 
+/* translation.c */
+
+LQT_EXTERN void lqt_translation_init();
+
+
 /* useratoms.c */
 
-uint8_t * quicktime_user_atoms_get_atom(quicktime_user_atoms_t * u, char * name, uint32_t * len);
+LQT_EXTERN uint8_t *
+quicktime_user_atoms_get_atom(quicktime_user_atoms_t * u,
+                              char * name, uint32_t * len);
 
-void quicktime_user_atoms_add_atom(quicktime_user_atoms_t * u,
-                                   char * name, uint8_t * data,
-                                   uint32_t len);
+LQT_EXTERN void quicktime_user_atoms_add_atom(quicktime_user_atoms_t * u,
+                                              char * name, uint8_t * data,
+                                              uint32_t len);
 
 void quicktime_user_atoms_read_atom(quicktime_t * file,
                                     quicktime_user_atoms_t * u,
@@ -961,48 +1166,73 @@ int quicktime_avg_chunk_samples(quicktime_t *file, quicktime_trak_t *trak);
 /* For seeking in CBR audio files: Get the chunk index for a specified sample.
    On return, chunk_sample will contain the first sample of the returned chunk */
 
-int quicktime_chunk_of_sample(int64_t *chunk_sample, 
-                              int64_t *chunk, 
-                              quicktime_trak_t *trak, 
-                              int64_t sample);
+LQT_EXTERN int quicktime_chunk_of_sample(int64_t *chunk_sample, 
+                                         int64_t *chunk, 
+                                         quicktime_trak_t *trak, 
+                                         int64_t sample);
 
 int64_t quicktime_chunk_to_offset(quicktime_t *file, quicktime_trak_t *trak, long chunk);
 
-long quicktime_offset_to_chunk(int64_t *chunk_offset, 
-                               quicktime_trak_t *trak, 
-                               int64_t offset);
 
 int64_t quicktime_sample_range_size(quicktime_trak_t *trak, 
                                     long chunk_sample, 
                                     long sample);
 int64_t quicktime_sample_to_offset(quicktime_t *file, quicktime_trak_t *trak, long sample);
 
-void quicktime_write_chunk_header(quicktime_t *file, 
-                                  quicktime_trak_t *trak, 
-                                  quicktime_atom_t *chunk);
-void quicktime_write_chunk_footer(quicktime_t *file, 
-                                  quicktime_trak_t *trak,
-                                  int current_chunk,
-                                  quicktime_atom_t *chunk, 
-                                  int samples);
+LQT_EXTERN void quicktime_write_chunk_header(quicktime_t *file, 
+                                             quicktime_trak_t *trak, 
+                                             quicktime_atom_t *chunk);
+LQT_EXTERN void quicktime_write_chunk_footer(quicktime_t *file, 
+                                             quicktime_trak_t *trak,
+                                             int current_chunk,
+                                             quicktime_atom_t *chunk, 
+                                             int samples);
 
 int quicktime_trak_duration(quicktime_trak_t *trak, 
-                            long *duration, 
-                            long *timescale);
+                            int64_t *duration, 
+                            int *timescale);
 
-int quicktime_trak_fix_counts(quicktime_t *file, quicktime_trak_t *trak);
-long quicktime_chunk_samples(quicktime_trak_t *trak, long chunk);
+int quicktime_trak_fix_counts(quicktime_t *file, quicktime_trak_t *trak,
+                              int moov_time_scale);
+LQT_EXTERN long quicktime_chunk_samples(quicktime_trak_t *trak, long chunk);
 int quicktime_trak_shift_offsets(quicktime_trak_t *trak, int64_t offset);
 
 
+/* tcmi.c */
+
+void quicktime_read_tcmi(quicktime_t *file, quicktime_tcmi_t *tcmi);
+void quicktime_write_tcmi(quicktime_t *file, quicktime_tcmi_t *tcmi);
+void quicktime_tcmi_dump(quicktime_tcmi_t *tcmi);
+void quicktime_tcmi_delete(quicktime_tcmi_t *tcmi);
+void quicktime_tcmi_init(quicktime_tcmi_t *tcmi);
+
+/* timecode.c */
+
+void lqt_flush_timecode(quicktime_t * file, int track, int64_t time,
+                        int force);
+
+
+/* tmcd.c */
+
+void quicktime_tmcd_dump(quicktime_tmcd_t *tmcd);
+void quicktime_tmcd_init(quicktime_tmcd_t *tmcd);
+void quicktime_tmcd_delete(quicktime_tmcd_t *tmcd);
+void quicktime_read_tmcd(quicktime_t *file, quicktime_tmcd_t *tmcd, quicktime_atom_t *parent_atom);
+void quicktime_write_tmcd(quicktime_t *file, quicktime_tmcd_t *tmcd);
+
+
 /* tref.c */
 
 int quicktime_tref_init(quicktime_tref_t *tref);
 int quicktime_tref_delete(quicktime_tref_t *tref);
 void quicktime_tref_dump(quicktime_tref_t *tref);
-int quicktime_read_tref(quicktime_t *file, quicktime_tref_t *tref);
+int quicktime_read_tref(quicktime_t *file, quicktime_tref_t *tref,
+                        quicktime_atom_t *tref_atom);
 void quicktime_write_tref(quicktime_t *file, quicktime_tref_t *tref);
 int quicktime_tref_init_qtvr(quicktime_tref_t *tref, int track_type);
+int quicktime_tref_init_chap(quicktime_tref_t * tref, int track_id);
+int quicktime_tref_init_tmcd(quicktime_tref_t * tref, int track_id);
+
 
 /* udta.c */
 
@@ -1012,6 +1242,13 @@ void quicktime_udta_dump(quicktime_udta_t *udta);
 int quicktime_read_udta(quicktime_t *file, quicktime_udta_t *udta, quicktime_atom_t *udta_atom);
 void quicktime_write_udta(quicktime_t *file, quicktime_udta_t *udta);
 int quicktime_set_udta_string(char **string, int *size, char *new_string);
+int quicktime_read_udta_string(quicktime_t *file,
+                               char **string,
+                               int *size, int ilst);
+
+int
+quicktime_write_udta_string(quicktime_t *file, const char * string,
+                            int ilst, lqt_charset_converter_t ** cnv);
 
 /* vmhd.c */
 
@@ -1061,8 +1298,8 @@ void quicktime_write_wave(quicktime_t *file, quicktime_wave_t *wave);
 void quicktime_wave_dump(quicktime_wave_t *wave);
 void quicktime_wave_delete(quicktime_wave_t *wave);
 
-uint8_t * quicktime_wave_get_user_atom(quicktime_trak_t * trak, char * name, uint32_t * len);
-void quicktime_wave_set_user_atom(quicktime_trak_t * trak, char * name, uint8_t * data,  uint32_t len);
+LQT_EXTERN uint8_t * quicktime_wave_get_user_atom(quicktime_trak_t * trak, char * name, uint32_t * len);
+LQT_EXTERN void quicktime_wave_set_user_atom(quicktime_trak_t * trak, char * name, uint8_t * data,  uint32_t len);
 
 
 
@@ -1074,8 +1311,8 @@ int quicktime_file_open(quicktime_t *file, const char *path, int rd, int wr);
 int quicktime_file_close(quicktime_t *file);
 int64_t quicktime_ftell(quicktime_t *file);
 int quicktime_fseek(quicktime_t *file, int64_t offset);
-int quicktime_read_data(quicktime_t *file, uint8_t *data, int64_t size);
-int quicktime_write_data(quicktime_t *file, uint8_t *data, int size);
+LQT_EXTERN int quicktime_read_data(quicktime_t *file, uint8_t *data, int64_t size);
+LQT_EXTERN int quicktime_write_data(quicktime_t *file, const uint8_t *data, int size);
 int64_t quicktime_byte_position(quicktime_t *file);
 void quicktime_read_pascal(quicktime_t *file, char *data);
 void quicktime_write_pascal(quicktime_t *file, char *data);
@@ -1114,12 +1351,14 @@ int quicktime_set_position(quicktime_t *file, int64_t position);
 void quicktime_copy_char32(char *output, char *input);
 void quicktime_print_chars(char *desc, uint8_t *input, int len);
 unsigned long quicktime_current_time(void);
-int quicktime_match_32(void *input, void *output);
+LQT_EXTERN int quicktime_match_32(void *input, void *output);
 int quicktime_match_24(char *input, char *output);
 
-void lqt_hexdump(uint8_t * data, int len, int linebreak);
+LQT_EXTERN void lqt_hexdump(uint8_t * data, int len, int linebreak);
 void lqt_hexdump_stdout(uint8_t * data, int len, int linebreak);
 
+void lqt_dump_time(uint64_t t);
+
 /* lqt_quicktime.c */
 
 int quicktime_init(quicktime_t *file);
@@ -1142,13 +1381,13 @@ int quicktime_delete_video_map(quicktime_video_map_t *vtrack);
 void quicktime_init_maps(quicktime_t * file);
 void lqt_update_frame_position(quicktime_video_map_t * track);
 
-void lqt_start_audio_vbr_chunk(quicktime_t * file, int track);
-void lqt_init_vbr_audio(quicktime_t * file, int track);
+LQT_EXTERN void lqt_start_audio_vbr_chunk(quicktime_t * file, int track);
+LQT_EXTERN void lqt_init_vbr_audio(quicktime_t * file, int track);
 
-int lqt_chunk_of_sample_vbr(int64_t *chunk_sample, 
-                            int64_t *chunk, 
-                            quicktime_trak_t *trak, 
-                            int64_t sample);
+LQT_EXTERN int lqt_chunk_of_sample_vbr(int64_t *chunk_sample, 
+                                       int64_t *chunk, 
+                                       quicktime_trak_t *trak, 
+                                       int64_t sample);
 
 /* Read all the information about the file. */
 /* Requires a MOOV atom be present in the file. */
@@ -1167,8 +1406,8 @@ void lqt_set_default_audio_parameters(quicktime_t * file, int track);
 /* Before and after writing subsequent frames, we must call
    quicktime_write_chunk_[header|footer] */
 
-void lqt_start_audio_vbr_frame(quicktime_t * file, int track);
-void lqt_finish_audio_vbr_frame(quicktime_t * file, int track, int num_samples);
+LQT_EXTERN void lqt_start_audio_vbr_frame(quicktime_t * file, int track);
+LQT_EXTERN void lqt_finish_audio_vbr_frame(quicktime_t * file, int track, int num_samples);
 
 /*
  *  Convenience function: Returns an array of chunk sizes
@@ -1192,6 +1431,17 @@ int quicktime_delete_vcodec(quicktime_video_map_t *vtrack);
 int quicktime_delete_acodec(quicktime_audio_map_t *vtrack);
 int quicktime_codecs_flush(quicktime_t *file);
 
+LQT_EXTERN void lqt_write_frame_header(quicktime_t * file, int track,
+                                       int pic_num1, int64_t pic_pts, int keyframe);
+
+LQT_EXTERN void lqt_write_frame_footer(quicktime_t * file, int track);
+
+void lqt_video_build_timestamp_tables(quicktime_t * file, int track);
+
+
+LQT_EXTERN void lqt_video_append_timestamp(quicktime_t * file, int track,
+                                           int64_t time, int duration);
+
 /* workarounds.c */
 
 int64_t quicktime_add3(int64_t a, int64_t b, int64_t c);
