@@ -1,10 +1,32 @@
-#include "funcprotos.h"
-#include <quicktime/colormodels.h>
-#include <quicktime/quicktime.h>
-#include <workarounds.h>
+/*******************************************************************************
+ v210.c
 
+ libquicktime - A library for reading and writing quicktime/avi/mp4 files.
+ http://libquicktime.sourceforge.net
+
+ Copyright (C) 2002 Heroine Virtual Ltd.
+ Copyright (C) 2002-2007 Members of the libquicktime project.
+
+ This library is free software; you can redistribute it and/or modify it under
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2.1 of the License, or (at your option)
+ any later version.
+
+ This library is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+ details.
+
+ You should have received a copy of the GNU Lesser General Public License along
+ with this library; if not, write to the Free Software Foundation, Inc., 51
+ Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*******************************************************************************/ 
+
+#include "lqt_private.h"
+#include "workarounds.h"
+#include "videocodec.h"
+#include <quicktime/colormodels.h>
 #include <stdlib.h>
-#include <string.h>
 
 typedef struct
     {
@@ -35,7 +57,7 @@ static int delete_codec(quicktime_video_map_t *vtrack)
 static void initialize(quicktime_video_map_t *vtrack, 
                quicktime_v210_codec_t *codec, int width, int height) 
     {
-
+        
     if  (codec->initialized != 0)
         return;
     codec->bytes_per_line = ((((width + 47) / 48) * 48 * 8) / 3);;
@@ -43,6 +65,7 @@ static void initialize(quicktime_video_map_t *vtrack,
 
     if  (!codec->buffer)
         codec->buffer = malloc(codec->buffer_alloc);
+    
     codec->initialized = 1;
     }
 
@@ -157,6 +180,9 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
         return 0;
         }
 
+    if(!codec->initialized)
+      lqt_set_fiel_uncompressed(file, track);
+    
     initialize(vtrack, codec, width, height);
 
     out_ptr = codec->buffer;
@@ -262,20 +288,19 @@ static int encode(quicktime_t *file, unsigned char **row_pointers, int track)
  * no padding will be performed.
 */
 	j = optr - out_ptr;
-        //        fprintf(stderr, "padding = %d\n", (int)(codec->bytes_per_line - j));
 	while (j++ < codec->bytes_per_line)
 	      *(optr++) = '\0';
         }
 
-    quicktime_write_chunk_header(file, trak, &chunk_atom);
+    lqt_write_frame_header(file, track,
+                           vtrack->current_position,
+                           -1, 0);
     result = !quicktime_write_data(file,
                                    codec->buffer,
                                    codec->bytes_per_line * height);
-    quicktime_write_chunk_footer(file,
-                                 trak,
-                                 vtrack->current_chunk,&chunk_atom,
-				 1);
-    vtrack->current_chunk++;
+
+    lqt_write_frame_footer(file, track);
+    
     return result;
     }
 
