@@ -1,9 +1,31 @@
-#include <funcprotos.h>
-#include <quicktime/quicktime.h>
+/*******************************************************************************
+ avi_ix.c
+
+ libquicktime - A library for reading and writing quicktime/avi/mp4 files.
+ http://libquicktime.sourceforge.net
+
+ Copyright (C) 2002 Heroine Virtual Ltd.
+ Copyright (C) 2002-2007 Members of the libquicktime project.
+
+ This library is free software; you can redistribute it and/or modify it under
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2.1 of the License, or (at your option)
+ any later version.
+
+ This library is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+ details.
+
+ You should have received a copy of the GNU Lesser General Public License along
+ with this library; if not, write to the Free Software Foundation, Inc., 51
+ Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*******************************************************************************/ 
+
+#include "lqt_private.h"
+#include <stdlib.h>
 #include <string.h>
 
-
-
 static char* make_tag(int number, char *tag)
 {
 	tag[0] = 'i';
@@ -43,7 +65,6 @@ void quicktime_update_ixtable(quicktime_t *file,
         quicktime_ix_t *ix = indx->table[indx->table_size-1].ix;
         quicktime_ixtable_t *ix_table;
 
-        //        fprintf(stderr, "quicktime_update_ixtable %d\n", trak->tkhd.track_id);
         
 /* Allocation */
 	if(ix->table_size >= ix->table_allocation)
@@ -142,15 +163,15 @@ void quicktime_read_ix(quicktime_t *file,
 void quicktime_ix_dump(quicktime_ix_t *ix)
   {
   int i;
-  printf("   table_size:      %d\n", ix->table_size);
-  printf("   longs_per_entry: %d\n", ix->longs_per_entry);
-  printf("   index_type:      %d\n", ix->index_type);
-  printf("   base_offset:     %lld\n", ix->base_offset);
-  printf("   tag:             %s\n", ix->tag);
-  printf("   chunk_id:        %s\n", ix->chunk_id);
+  lqt_dump("   table_size:      %d\n", ix->table_size);
+  lqt_dump("   longs_per_entry: %d\n", ix->longs_per_entry);
+  lqt_dump("   index_type:      %d\n", ix->index_type);
+  lqt_dump("   base_offset:     %"PRId64"\n", ix->base_offset);
+  lqt_dump("   tag:             %s\n", ix->tag);
+  lqt_dump("   chunk_id:        %s\n", ix->chunk_id);
   for(i = 0; i < ix->table_size; i++)
     {
-    printf("   off: %d, size: %d, k: %d\n", ix->table[i].relative_offset,
+    lqt_dump("   off: %d, size: %d, k: %d\n", ix->table[i].relative_offset,
            (ix->table[i].size & 0x7FFFFFFF),
            !(ix->table[i].size & 0x80000000));
     }
