@@ -1,7 +1,32 @@
-#include <quicktime/quicktime.h>
-#include <funcprotos.h>
+/*******************************************************************************
+ avi_riff.c
+
+ libquicktime - A library for reading and writing quicktime/avi/mp4 files.
+ http://libquicktime.sourceforge.net
+
+ Copyright (C) 2002 Heroine Virtual Ltd.
+ Copyright (C) 2002-2007 Members of the libquicktime project.
+
+ This library is free software; you can redistribute it and/or modify it under
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2.1 of the License, or (at your option)
+ any later version.
+
+ This library is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+ details.
+
+ You should have received a copy of the GNU Lesser General Public License along
+ with this library; if not, write to the Free Software Foundation, Inc., 51
+ Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*******************************************************************************/ 
+
+#include "lqt_private.h"
+#include <stdlib.h>
 #include <string.h>
 
+#define LOG_DOMAIN "avi_riff"
 
 void quicktime_read_riff(quicktime_t *file, quicktime_atom_t *parent_atom)
   {
@@ -14,7 +39,6 @@ void quicktime_read_riff(quicktime_t *file, quicktime_atom_t *parent_atom)
 
   // AVI
   quicktime_read_data(file, data, 4);
-  //printf("quicktime_read_riff 1 %llx\n", quicktime_position(file));
 
   // Certain AVI parameters must be copied over to quicktime objects:
   // hdrl -> moov
@@ -24,15 +48,6 @@ void quicktime_read_riff(quicktime_t *file, quicktime_atom_t *parent_atom)
     {
     result = quicktime_atom_read_header(file, &leaf_atom);
 
-    /*
-     * printf("quicktime_read_riff 1 %llx %llx %c%c%c%c\n", 
-     * leaf_atom.start,
-     * leaf_atom.size,
-     * leaf_atom.type[0], 
-     * leaf_atom.type[1], 
-     * leaf_atom.type[2], 
-     * leaf_atom.type[3]);
-     */
     if(!result)
       {
       if(quicktime_atom_is(&leaf_atom, "LIST"))
@@ -46,16 +61,12 @@ void quicktime_read_riff(quicktime_t *file, quicktime_atom_t *parent_atom)
           if(quicktime_match_32(data, "hdrl"))
             {
             // No size here.
-            //printf("quicktime_read_riff 10 %llx\n", quicktime_position(file));
             quicktime_read_hdrl(file, &riff->hdrl, &leaf_atom);
-            //printf("quicktime_read_riff 20 %llx\n", quicktime_position(file));
             riff->have_hdrl = 1;
             }
           else if(quicktime_match_32(data, "movi"))  // Got LIST 'movi'
             {
-            //printf("quicktime_read_riff 30 %llx\n", quicktime_position(file));
             quicktime_read_movi(file, &leaf_atom, &riff->movi);
-            //printf("quicktime_read_riff 40 %llx\n", quicktime_position(file));
             }
           else if(quicktime_match_32(data, "INFO"))
             {
@@ -79,7 +90,6 @@ void quicktime_read_riff(quicktime_t *file, quicktime_atom_t *parent_atom)
         else if(quicktime_atom_is(&leaf_atom, "idx1"))
           {
           
-          //printf("quicktime_read_riff 50 %llx\n", quicktime_position(file));
           // Preload idx1 here
           int64_t start_position = quicktime_position(file);
           long temp_size = leaf_atom.end - start_position;
@@ -92,7 +102,6 @@ void quicktime_read_riff(quicktime_t *file, quicktime_atom_t *parent_atom)
 
           // Read idx1
           quicktime_read_idx1(file, riff, &leaf_atom);
-          //printf("quicktime_read_riff 60 %llx\n", quicktime_position(file));
 
           }
         else if(quicktime_atom_is(&leaf_atom, "INFO"))
@@ -110,7 +119,6 @@ void quicktime_read_riff(quicktime_t *file, quicktime_atom_t *parent_atom)
       }
     }while(!result && quicktime_position(file) < parent_atom->end);
     
-  //printf("quicktime_read_riff 10\n");
   
   }
 
@@ -119,8 +127,8 @@ quicktime_riff_t* quicktime_new_riff(quicktime_t *file)
 {
 	if(file->total_riffs >= MAX_RIFFS)
 	{
-		fprintf(stderr, "quicktime_new_riff file->total_riffs >= MAX_RIFFS\n");
-		return 0;
+        lqt_log(file, LQT_LOG_ERROR, LOG_DOMAIN, "file->total_riffs >= MAX_RIFFS");
+        return 0;
 	}
 	else
 	{
@@ -181,14 +189,11 @@ void quicktime_init_riff(quicktime_t *file)
   
   if(file->file_type == LQT_FILE_AVI_ODML)
     {
-    //    fprintf(stderr, "init_riff (ODML)\n");
     for(i = 0; i < file->moov.total_tracks; i++)
       {
       quicktime_indx_init_riff(file, file->moov.trak[i]);
       }
     }
-  //  else
-  //    fprintf(stderr, "init_riff (AVI 1.0)\n");
   }
 
 void quicktime_finalize_riff(quicktime_t *file, quicktime_riff_t *riff)
@@ -206,16 +211,12 @@ void quicktime_finalize_riff(quicktime_t *file, quicktime_riff_t *riff)
   quicktime_finalize_movi(file, &riff->movi);
   if(riff->have_hdrl)
     {
-    //    printf("quicktime_finalize_riff 1\n");
     quicktime_finalize_hdrl(file, &riff->hdrl);
-    //    printf("quicktime_finalize_riff 10\n");
 
     // Write original index for first RIFF
     quicktime_write_idx1(file, &riff->idx1);
-    //    printf("quicktime_finalize_riff 100\n");
     }
   quicktime_atom_write_footer(file, &riff->atom);
-  //  printf("quicktime_finalize_riff\n");
   }
 
 
@@ -284,14 +285,10 @@ static void insert_audio_packet(quicktime_trak_t * trak,
   quicktime_stco_t *stco;
   quicktime_stsc_t *stsc;
   quicktime_stts_t *stts = &trak->mdia.minf.stbl.stts;
-#if 0
-  fprintf(stderr, "insert_audio_packet: %lld, %d\n",
-          offset, size);
-#endif  
   /* Update stco */
 
   stco = &trak->mdia.minf.stbl.stco;
-  quicktime_update_stco(stco, stco->total_entries + 1, offset);
+  quicktime_update_stco(stco, stco->total_entries, offset);
 
   /* Update stsc */
   
@@ -302,9 +299,9 @@ static void insert_audio_packet(quicktime_trak_t * trak,
   /* stsc will be initialized with 1 entry and zero samples */
 
   if(stsc->table[0].samples == 0)
-    quicktime_update_stsc(stsc, 1, num_samples);
+    quicktime_update_stsc(stsc, 0, num_samples);
   else
-    quicktime_update_stsc(stsc, stsc->total_entries+1, num_samples);
+    quicktime_update_stsc(stsc, stsc->total_entries, num_samples);
 
   /* Update total samples */
 
@@ -345,7 +342,7 @@ static void insert_video_packet(quicktime_trak_t * trak,
   
   /* Update stco */
   
-  quicktime_update_stco(stco, stco->total_entries + 1, offset);
+  quicktime_update_stco(stco, stco->total_entries, offset);
 
   /* Update stss */
 
@@ -434,7 +431,7 @@ static void indx_build_index(quicktime_t *file)
         if(trak->mdia.minf.is_audio)
           insert_audio_packet(trak,
                               ix->base_offset + ix->table[k].relative_offset,
-                              ix->table[k].size);
+                              ix->table[k].size & 0x7FFFFFFF);
         
         else if(trak->mdia.minf.is_video)
           insert_video_packet(trak,
@@ -497,5 +494,7 @@ void quicktime_riff_dump(quicktime_riff_t * riff)
       quicktime_strl_dump(riff->hdrl.strl[i]);
       i++;
       }
+    if(riff->idx1.table_size)
+      quicktime_idx1_dump(&riff->idx1);
     }
   }
