@@ -1,6 +1,29 @@
-#include <stdio.h>
-#include <funcprotos.h>
-#include <quicktime/quicktime.h>
+/*******************************************************************************
+ ctab.c
+
+ libquicktime - A library for reading and writing quicktime/avi/mp4 files.
+ http://libquicktime.sourceforge.net
+
+ Copyright (C) 2002 Heroine Virtual Ltd.
+ Copyright (C) 2002-2007 Members of the libquicktime project.
+
+ This library is free software; you can redistribute it and/or modify it under
+ the terms of the GNU Lesser General Public License as published by the Free
+ Software Foundation; either version 2.1 of the License, or (at your option)
+ any later version.
+
+ This library is distributed in the hope that it will be useful, but WITHOUT
+ ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+ details.
+
+ You should have received a copy of the GNU Lesser General Public License along
+ with this library; if not, write to the Free Software Foundation, Inc., 51
+ Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+*******************************************************************************/ 
+
+#include "lqt_private.h"
+#include <stdlib.h>
 
 int quicktime_ctab_init(quicktime_ctab_t *ctab)
 {
@@ -26,18 +49,18 @@ int quicktime_ctab_delete(quicktime_ctab_t *ctab)
 void quicktime_ctab_dump(quicktime_ctab_t *ctab)
 {
 	int i;
-	printf(" color table\n");
-	printf("  seed %ld\n", ctab->seed);
-	printf("  flags %ld\n", ctab->flags);
-	printf("  size %ld\n", ctab->size);
-	printf("  colors ");
+	lqt_dump(" color table (ctab)\n");
+	lqt_dump("  seed %ld\n", ctab->seed);
+	lqt_dump("  flags %ld\n", ctab->flags);
+	lqt_dump("  size %ld\n", ctab->size);
+	lqt_dump("  colors ");
 	for(i = 0; i < ctab->size; i++)
 	{
-		printf("[0x%02x 0x%02x 0x%02x 0x%02x]\n",
+		lqt_dump("[0x%02x 0x%02x 0x%02x 0x%02x]\n",
                        (ctab->red[i])>>8, (ctab->green[i])>>8,
                        (ctab->blue[i])>>8, (ctab->alpha[i])>>8);
 	}
-	printf("\n");
+	lqt_dump("\n");
 }
 
 int quicktime_read_ctab(quicktime_t *file, quicktime_ctab_t *ctab)
